@@ -17,8 +17,8 @@ import math
 import click
 import pandas as pd
 import time
-from selenium import webdriver
 import selenium
+from selenium import webdriver
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support.ui import Select
 from selenium.webdriver.chrome.options import Options 
@@ -35,10 +35,10 @@ pd.set_option('display.max_rows', 100)
 ## COMMAND LINE INTERFACE
 
 @click.group()
-@click.version_option("76.2.5", package_name="alacorder")
+@click.version_option("76.4.5", package_name="alacorder")
 def cli():
     """
-    ALACORDER beta 76
+    ALACORDER beta 76.4
 
     Alacorder processes case detail PDFs into data tables suitable for research purposes. Alacorder also generates compressed text archives from the source PDFs to speed future data collection from the same set of cases.
 
@@ -296,13 +296,13 @@ def archive(input_path, output_path, count, overwrite, dedupe, log, no_write, no
 
     # finalize config
 
-
-
 # SCRAPER
 
 def readPartySearchQuery(path, qmax=0, qskip=0, speed=1, no_log=False):
     good = os.path.exists(path)
-
+    """
+     Use headers NAME, PARTY_TYPE, SSN, DOB, COUNTY, DIVISION, CASE_YEAR, NO_RECORDS, and FILED_BEFORE in an Excel spreadsheet to submit a list of queries for Alacorder to scrape.
+    """
     ext = os.path.splitext(path)[1]
     if ext == ".xlsx" or ".xls":
         query = pd.read_excel(path, dtype=pd.StringDtype())
@@ -320,25 +320,20 @@ def readPartySearchQuery(path, qmax=0, qskip=0, speed=1, no_log=False):
         writer_df['RETRIEVED_ON'] = pd.NaT
         writer_df['CASES_FOUND'] = pd.NaT
 
-
-    query_out = pd.DataFrame(columns=["NAME", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER"])
+    query_out = pd.DataFrame(columns=["NAME", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER", "RETRIEVED_ON", "CASES_FOUND"])
 
     clist = []
     for c in query.columns:
-        if c.upper().strip().replace(" ","_") in ["NAME", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER"]:
+        if c.upper().strip().replace(" ","_") in ["NAME", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER", "RETRIEVED_ON", "CASES_FOUND"]:
             clist += [c]
             query_out[c.upper().strip().replace(" ","_")] = query[c]
-            writer_df[c.upper().strip().replace(" ","_")] = query[c]
-    if not no_log:
-        click.echo(f"Search field column \'{c.upper()}\' identified in query file. Use headers NAME, PARTY_TYPE, SSN, DOB, COUNTY, DIVISION, CASE_YEAR, NO_RECORDS, and FILED_BEFORE in an Excel spreadsheet to submit a list of queries for Alacorder to scrape.")
+            if not no_log:
+                click.echo(f"Search field column {c.upper()} identified in query file.")
+
 
     query_out = query_out.fillna('')
     return [query_out, writer_df]
 
-
-
-
-
 @cli.command(help="Search Alacourt.com with query template (see /templates on github)")
 @click.option("--input-path", "-in", "listpath", required=True, prompt="Path to query table", help="Path to query table/spreadsheet (.xls, .xlsx, .csv, .json)", type=click.Path())
 @click.option("--output-path", "-out", "path", required=True, prompt="PDF download path", type=click.Path(), help="Desired PDF output directory")
@@ -347,11 +342,12 @@ def readPartySearchQuery(path, qmax=0, qskip=0, speed=1, no_log=False):
 @click.option("--password", "-p","pwd", required=True, prompt="Alacourt Password", help="Password on Alacourt.com", hide_input=True)
 @click.option("--max", "-max","qmax", required=False, type=int, help="Maximum queries to conduct on Alacourt.com",default=0)
 @click.option("--skip", "-skip", "qskip", required=False, type=int, help="Skip entries at top of query file",default=0)
-@click.option("--speed", default=1, type=int, help="Speed multiplier")
+@click.option("--speed", default=1, type=float, help="Speed multiplier")
 @click.option("--no-log","-nl", is_flag=True, default=False, help="Do not print logs to console")
 @click.option("--no-update","-w", is_flag=True, default=False, help="Do not update query template after completion")
 @click.option("--ignore-complete","-g", is_flag=True, default=False, help="Ignore initial completion status in query template")
-def scrape(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update, ignore_complete):
+@click.option("--debug","-d", is_flag=True, default=False, help="Print detailed runtime information to console")
+def scrape(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update, ignore_complete, debug):
     """
     Use headers NAME, PARTY_TYPE, SSN, DOB, COUNTY, DIVISION, CASE_YEAR, NO_RECORDS, and FILED_BEFORE in an Excel spreadsheet to submit a list of queries for Alacorder to scrape.
 
@@ -359,17 +355,14 @@ def scrape(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update,
     KEEP YOUR COMPUTER POWERED ON AND CONNECTED TO THE INTERNET.
     SET DEFAULT DOWNLOADS DIRECTORY IN BROWSER TO DESIRED PDF DIRECTORY TARGET BEFORE INITIATING TASK.
     """
+    sys.tracebacklimit = 10
     rq = readPartySearchQuery(listpath, qmax, qskip, no_log)
 
-    query = rq[0] # for scraper - only search columns
-    query_writer = rq[1] # original sheet for write completion 
-
-    if not ignore_complete:
-        try:
-            incomplete = query.RETRIEVED_ON.map(lambda x: pd.isnull(x))
-            query = query[incomplete]
-        except:
-            pass
+    query = pd.DataFrame(rq[0]) # for scraper - only search columns
+    query_writer = pd.DataFrame(rq[1]) # original sheet for write completion 
+    incomplete = query.RETRIEVED_ON.map(lambda x: True if x == "" else False)
+    query = query[incomplete]
+    # query = query.reindex()
 
     options = webdriver.ChromeOptions()
     options.add_experimental_option('prefs', {
@@ -389,19 +382,22 @@ def scrape(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update,
 
     if not no_log:
         cal.echo_green("Authentication successful. Fetching cases via party search...")
-    
-
 
     for i, n in enumerate(query.index):
+        if debug:
+            click.echo(driver.current_url)
         if driver.current_url == "https://v2.alacourt.com/frmlogin.aspx":
                 login(driver, cID, uID, pwd, speed, no_log)
+        driver.implicitly_wait(2/speed)
         results = party_search(driver, name=query.NAME[n], party_type=query.PARTY_TYPE[n], ssn=query.SSN[n], dob=query.DOB[n], county=query.COUNTY[n], division=query.DIVISION[n], case_year=query.CASE_YEAR[n], no_records=query.NO_RECORDS[n], filed_before=query.FILED_BEFORE[n], filed_after=query.FILED_AFTER[n], speed=speed, no_log=no_log)
+        driver.implicitly_wait(2/speed)
         if len(results) == 0:
+            query_writer['RETRIEVED_ON'][n] = str(math.floor(time.time()))
+            query_writer['CASES_FOUND'][n] = "0"
             if not no_log:
-                click.echo(f"Skipped {query.NAME[n]} due to unknown issue...")
+                click.echo(f"Found no results for query: {query.NAME[n]}")
             continue
-        time.sleep(3)
-        with click.progressbar(results, show_eta=False, label=f"{i+1}/{query.index.stop+1}: {query.NAME[n]}") as bar:
+        with click.progressbar(results, show_eta=False, label=f"#{n}: {query.NAME[n]}") as bar:
             for url in bar:
                 downloadPDF(driver, url)
                 driver.implicitly_wait(0.5/speed)
@@ -409,10 +405,7 @@ def scrape(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update,
         if not no_update:
             query_writer['RETRIEVED_ON'][n] = str(math.floor(time.time()))
             query_writer['CASES_FOUND'][n] = str(len(results))
-            query_writer.to_excel(listpath)
-
-
-
+            query_writer.to_excel(listpath,sheet_name="PartySearchQuery",index=False)
 
 def login(driver, cID, username, pwd, speed, no_log=False):
 
@@ -458,9 +451,10 @@ def login(driver, cID, username, pwd, speed, no_log=False):
 
     driver.implicitly_wait(0.5/speed)
 
-def party_search(driver, name = "", party_type = "", ssn="", dob="", county="", division="", case_year="", no_records="", filed_before="", filed_after="", speed=1.25, no_log=False):
+def party_search(driver, name = "", party_type = "", ssn="", dob="", county="", division="", case_year="", no_records="", filed_before="", filed_after="", speed=1, no_log=False, debug=False):
+
+    speed = speed * 1.5
 
-    driver.implicitly_wait(1/speed)
 
     if "frmIndexSearchForm" not in driver.current_url:
         driver.get("https://v2.alacourt.com/frmIndexSearchForm.aspx")
@@ -476,22 +470,20 @@ def party_search(driver, name = "", party_type = "", ssn="", dob="", county="",
             cal.echo_red("Connection error. Attempting reconnection...")
         driver.refresh()
         driver.implicitly_wait(10/speed)
-        try:
-            party_name_box = driver.find_element(by=By.NAME,value="ctl00$ContentPlaceHolder1$txtName")
-            if not no_log:
-                cal.echo_green("Successfully connected and logged into Alacourt!")
+        party_name_box = driver.find_element(by=By.NAME,value="ctl00$ContentPlaceHolder1$txtName")
+        if not no_log:
+            cal.echo_green("Successfully connected and logged into Alacourt!")
 
     # field search
 
     if name != "":
         party_name_box.send_keys(name)
     if ssn != "":
-        ssn_box.send_keys(ssn)
         ssn_box = driver.find_element(by=By.NAME, value="ctl00$ContentPlaceHolder1$txtSSN")
+        ssn_box.send_keys(ssn)
     if dob != "":
-        date_of_birth_box.send_keys(dob)
         date_of_birth_box = driver.find_element(by=By.NAME,value="ctl00$ContentPlaceHolder1$txtDOB")
-
+        date_of_birth_box.send_keys(dob)
     if party_type != "":
         party_type_select = driver.find_element(by=By.NAME, value="ctl00$ContentPlaceHolder1$rdlPartyType")
         if party_type == "plaintiffs":
@@ -524,24 +516,50 @@ def party_search(driver, name = "", party_type = "", ssn="", dob="", county="",
         filed_after_box = driver.find_element(by=By.NAME, value="ctl00$ContentPlaceHolder1$txtTo")
         filed_after_box.send_keys(sfiled_after)
 
+    driver.implicitly_wait(1/speed)
+
     # submit search
     search_button = driver.find_element(by=By.ID,value="searchButton")
-    search_button.click()
+
+    driver.implicitly_wait(1/speed)
+    try:
+        search_button.click()
+    except:
+        driver.implicitly_wait(5/speed)
+        time.sleep(10)
+
+    if debug:
+        click.echo("Submitted party search form...")
+
+    driver.implicitly_wait(1/speed)
 
     # count pages
-    driver.implicitly_wait(0.5/speed)
-
     try:
         page_counter = driver.find_element(by=By.ID,value="ContentPlaceHolder1_dg_tcPageXofY").text
         pages = int(page_counter.strip()[-1])
+        Results
+
     except:
         pages = 1
 
+    # count results
+    try:
+        results_indicator = driver.find_element(by=By.ID, value="ContentPlaceHolder1_lblResultCount")
+        results_count = int(results_indicator.text.replace("Search Results: ","").replace(" records returned.","").strip())
+    except:
+        pass
+
+    if debug:
+        click.echo(f"Found {results_count} results, fetching URLs and downloading PDFs...")
+
+    if debug:
+        click.echo(pages)
 
     # get PDF links from each page
     pdflinks = []
     i = 0
-    for i in range(0,pages-1):
+    for i in range(0,pages):
+        driver.implicitly_wait(0.5/speed)
         hovers = driver.find_elements(By.CLASS_NAME, "menuHover")
         for x in hovers:
             try:
@@ -567,8 +585,7 @@ def party_search(driver, name = "", party_type = "", ssn="", dob="", county="",
 
 def downloadPDF(driver, url, speed=1, no_log=False):
     a = driver.get(url)
-    driver.implicitly_wait(1/speed)
-    time.sleep(1/speed)
+    driver.implicitly_wait(.5/speed)
 
 
 if __name__ == "__main__":
